@@ -23,6 +23,7 @@
 #include "config.h"
 #include "gfilterinputstream.h"
 #include "ginputstream.h"
+#include "gsimpleasyncresult.h"
 #include "glibintl.h"
 
 #include "gioalias.h"
@@ -36,7 +37,8 @@
 
 enum {
   PROP_0,
-  PROP_BASE_STREAM
+  PROP_BASE_STREAM,
+  PROP_CLOSE_BASE
 };
 
 static void     g_filter_input_stream_set_property (GObject      *object,
@@ -63,36 +65,18 @@ static gssize   g_filter_input_stream_skip         (GInputStream         *stream
 static gboolean g_filter_input_stream_close        (GInputStream         *stream,
                                                     GCancellable         *cancellable,
                                                     GError              **error);
-static void     g_filter_input_stream_read_async   (GInputStream         *stream,
-                                                    void                 *buffer,
-                                                    gsize                 count,
-                                                    int                   io_priority,
-                                                    GCancellable         *cancellable,
-                                                    GAsyncReadyCallback   callback,
-                                                    gpointer              user_data);
-static gssize   g_filter_input_stream_read_finish  (GInputStream         *stream,
-                                                    GAsyncResult         *result,
-                                                    GError              **error);
-static void     g_filter_input_stream_skip_async   (GInputStream         *stream,
-                                                    gsize                 count,
-                                                    int                   io_priority,
-                                                    GCancellable         *cancellabl,
-                                                    GAsyncReadyCallback   callback,
-                                                    gpointer              datae);
-static gssize   g_filter_input_stream_skip_finish  (GInputStream         *stream,
-                                                    GAsyncResult         *result,
-                                                    GError              **error);
-static void     g_filter_input_stream_close_async  (GInputStream         *stream,
-                                                    int                   io_priority,
-                                                    GCancellable         *cancellabl,
-                                                    GAsyncReadyCallback   callback,
-                                                    gpointer              data);
-static gboolean g_filter_input_stream_close_finish (GInputStream         *stream,
-                                                    GAsyncResult         *result,
-                                                    GError              **error);
 
 G_DEFINE_TYPE (GFilterInputStream, g_filter_input_stream, G_TYPE_INPUT_STREAM)
 
+#define GET_PRIVATE(inst) G_TYPE_INSTANCE_GET_PRIVATE (inst, \
+  G_TYPE_FILTER_INPUT_STREAM, GFilterInputStreamPrivate)
+
+typedef struct
+{
+  gboolean close_base;
+  GAsyncReadyCallback outstanding_callback;
+  gpointer outstanding_user_data;
+} GFilterInputStreamPrivate;
 
 static void
 g_filter_input_stream_class_init (GFilterInputStreamClass *klass)
@@ -110,22 +94,24 @@ g_filter_input_stream_class_init (GFilterInputStreamClass *klass)
   istream_class->skip  = g_filter_input_stream_skip;
   istream_class->close_fn = g_filter_input_stream_close;
 
-  istream_class->read_async   = g_filter_input_stream_read_async;
-  istream_class->read_finish  = g_filter_input_stream_read_finish;
-  istream_class->skip_async   = g_filter_input_stream_skip_async;
-  istream_class->skip_finish  = g_filter_input_stream_skip_finish;
-  istream_class->close_async  = g_filter_input_stream_close_async;
-  istream_class->close_finish = g_filter_input_stream_close_finish;
+  g_type_class_add_private (klass, sizeof (GFilterInputStreamPrivate));
 
   g_object_class_install_property (object_class,
                                    PROP_BASE_STREAM,
                                    g_param_spec_object ("base-stream",
                                                          P_("The Filter Base Stream"),
-                                                         P_("The underlying base stream the io ops will be done on"),
+                                                         P_("The underlying base stream on which the io ops will be done."),
                                                          G_TYPE_INPUT_STREAM,
                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | 
                                                          G_PARAM_STATIC_NAME|G_PARAM_STATIC_NICK|G_PARAM_STATIC_BLURB));
 
+  g_object_class_install_property (object_class,
+                                   PROP_CLOSE_BASE,
+                                   g_param_spec_boolean ("close-base-stream",
+                                                         P_("Close Base Stream"),
+                                                         P_("If the base stream should be closed when the filter stream is closed."),
+                                                         TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | 
+                                                         G_PARAM_STATIC_NAME|G_PARAM_STATIC_NICK|G_PARAM_STATIC_BLURB));
 }
 
 static void
@@ -146,6 +132,11 @@ g_filter_input_stream_set_property (GObject         *object,
       filter_stream->base_stream = G_INPUT_STREAM (obj); 
       break;
 
+    case PROP_CLOSE_BASE:
+      g_filter_input_stream_set_close_base_stream (filter_stream,
+                                                   g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -169,6 +160,10 @@ g_filter_input_stream_get_property (GObject    *object,
       g_value_set_object (value, filter_stream->base_stream);
       break;
 
+    case PROP_CLOSE_BASE:
+      g_value_set_boolean (value, GET_PRIVATE (filter_stream)->close_base);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -210,6 +205,49 @@ g_filter_input_stream_get_base_stream (GFilterInputStream *stream)
   return stream->base_stream;
 }
 
+/**
+ * g_filter_input_stream_get_close_base_stream:
+ * @stream: a #GFilterInputStream.
+ *
+ * Returns whether the base stream will be closed when @stream is
+ * closed.
+ *
+ * Return value: %TRUE if the base stream will be closed.
+ **/
+gboolean
+g_filter_input_stream_get_close_base_stream (GFilterInputStream *stream)
+{
+  g_return_val_if_fail (G_IS_FILTER_INPUT_STREAM (stream), FALSE);
+
+  return GET_PRIVATE (stream)->close_base;
+}
+
+/**
+ * g_filter_input_stream_set_close_base_stream:
+ * @stream: a #GFilterInputStream.
+ * @close_base: %TRUE to close the base stream.
+ *
+ * Sets whether the base stream will be closed when @stream is closed.
+ **/
+void
+g_filter_input_stream_set_close_base_stream (GFilterInputStream *stream,
+                                             gboolean            close_base)
+{
+  GFilterInputStreamPrivate *priv;
+
+  g_return_if_fail (G_IS_FILTER_INPUT_STREAM (stream));
+
+  close_base = !!close_base;
+ 
+  priv = GET_PRIVATE (stream);
+
+  if (priv->close_base != close_base)
+    {
+      priv->close_base = close_base;
+      g_object_notify (G_OBJECT (stream), "close-base-stream");
+    }
+}
+
 static gssize
 g_filter_input_stream_read (GInputStream  *stream,
                             void          *buffer,
@@ -258,140 +296,20 @@ g_filter_input_stream_close (GInputStream  *stream,
                              GCancellable  *cancellable,
                              GError       **error)
 {
-  GFilterInputStream *filter_stream;
-  GInputStream       *base_stream;
-  gboolean            res;
+  gboolean res = TRUE;
 
-  filter_stream = G_FILTER_INPUT_STREAM (stream);
-  base_stream = filter_stream->base_stream;
+  if (GET_PRIVATE (stream)->close_base)
+    {
+      GFilterInputStream *filter_stream;
+      GInputStream       *base_stream;
 
-  res = g_input_stream_close (base_stream,
-                              cancellable,
-                              error);
+      filter_stream = G_FILTER_INPUT_STREAM (stream);
+      base_stream = filter_stream->base_stream;
 
-  return res;
-}
-
-static void
-g_filter_input_stream_read_async (GInputStream        *stream,
-                                  void                *buffer,
-                                  gsize                count,
-                                  int                  io_priority,
-                                  GCancellable        *cancellable,
-                                  GAsyncReadyCallback  callback,
-                                  gpointer             user_data)
-{
-  GFilterInputStream *filter_stream;
-  GInputStream       *base_stream;
-
-  filter_stream = G_FILTER_INPUT_STREAM (stream);
-  base_stream = filter_stream->base_stream;
-
-  g_input_stream_read_async (base_stream,
-                             buffer,
-                             count,
-                             io_priority,
-                             cancellable,
-                             callback,
-                             user_data);
-}
-
-static gssize
-g_filter_input_stream_read_finish (GInputStream  *stream,
-                                   GAsyncResult  *result,
-                                   GError       **error)
-{
-  GFilterInputStream *filter_stream;
-  GInputStream       *base_stream;
-  gssize nread;
-
-  filter_stream = G_FILTER_INPUT_STREAM (stream);
-  base_stream = filter_stream->base_stream;
-
-  nread = g_input_stream_read_finish (base_stream,
-                                      result,
-                                      error);
-  
-  return nread;
-}
-
-static void
-g_filter_input_stream_skip_async (GInputStream        *stream,
-                                  gsize                count,
-                                  int                  io_priority,
-                                  GCancellable        *cancellable,
-                                  GAsyncReadyCallback  callback,
-                                  gpointer             user_data)
-{
-  GFilterInputStream *filter_stream;
-  GInputStream       *base_stream;
-
-  filter_stream = G_FILTER_INPUT_STREAM (stream);
-  base_stream = filter_stream->base_stream;
-
-  g_input_stream_skip_async (base_stream,
-                             count,
-                             io_priority,
-                             cancellable,
-                             callback,
-                             user_data);
-
-}
-
-static gssize
-g_filter_input_stream_skip_finish (GInputStream  *stream,
-                                   GAsyncResult  *result,
-                                   GError       **error)
-{
-  GFilterInputStream *filter_stream;
-  GInputStream       *base_stream;
-  gssize nskipped;
-
-  filter_stream = G_FILTER_INPUT_STREAM (stream);
-  base_stream = filter_stream->base_stream;
-
-  nskipped = g_input_stream_skip_finish (base_stream,
-                                         result,
-                                         error);
-
-  return nskipped;
-}
-
-static void
-g_filter_input_stream_close_async (GInputStream        *stream,
-                                   int                  io_priority,
-                                   GCancellable        *cancellable,
-                                   GAsyncReadyCallback  callback,
-                                   gpointer             user_data)
-{
-  GFilterInputStream *filter_stream;
-  GInputStream       *base_stream;
-
-  filter_stream = G_FILTER_INPUT_STREAM (stream);
-  base_stream = filter_stream->base_stream;
-
-  g_input_stream_close_async (base_stream,
-                              io_priority,
-                              cancellable,
-                              callback,
-                              user_data);
-}
-
-static gboolean
-g_filter_input_stream_close_finish (GInputStream  *stream,
-                                    GAsyncResult  *result,
-                                    GError       **error)
-{
-  GFilterInputStream *filter_stream;
-  GInputStream       *base_stream;
-  gboolean res;
-
-  filter_stream = G_FILTER_INPUT_STREAM (stream);
-  base_stream = filter_stream->base_stream;
-
-  res = g_input_stream_close_finish (stream,
-                                     result,
-                                     error);
+      res = g_input_stream_close (base_stream,
+                                  cancellable,
+                                  error);
+    }
 
   return res;
 }
