@@ -788,10 +788,7 @@ _g_local_file_info_get_parent_info (const char            *dir,
 				    GFileAttributeMatcher *attribute_matcher,
 				    GLocalParentFileInfo  *parent_info)
 {
-  /* Use plain struct stat for now as long as we only look at the
-   * S_ISVTX bit which doesn't exist on Win32 anyway.
-   */
-  struct stat statbuf;
+  GStatBuf statbuf;
   int res;
 
   parent_info->extra_data = NULL;
@@ -1432,6 +1429,7 @@ _g_local_file_info_get (const char             *basename,
   char *symlink_target;
   GVfs *vfs;
   GVfsClass *class;
+  guint64 device;
 
   info = g_file_info_new ();
 
@@ -1483,7 +1481,9 @@ _g_local_file_info_get (const char             *basename,
       g_free (display_name);
       return NULL;
     }
-  
+
+  device = statbuf.st_dev;
+
 #ifdef S_ISLNK
   is_symlink = S_ISLNK (statbuf.st_mode);
 #else
@@ -1711,7 +1711,7 @@ _g_local_file_info_get (const char             *basename,
     {
       class->local_file_add_info (vfs,
                                   path,
-                                  statbuf.st_dev,
+                                  device,
                                   attribute_matcher,
                                   info,
                                   NULL,
